/*
 * (C) Copyright The University of Manchester 2004
 */
package com.ibm.JikesRVM;
import org.binarytranslator.PPC_Emulator.*;

/**
 * This class provides the bridge between the Java compiled world and
 * the world compiled using the PPC emulator.  VM_Uninterruptible is
 * used to prevent garbage collection errors with the dynamic bridge
 * code.
 */
public class DynamicCodeRunner implements VM_DynamicBridge, VM_Uninterruptible {
  /**
	* The bridge into the PPC emulator code
	*
	* @param code the code to be executed
	* @param ps the process space the code will work upon
	* @return the code will return the PC value of the next instruction
	*/
  public static int invokeCode (VM_CodeArray code, PPC_ProcessSpace ps) throws BadInstructionException, VM_PragmaNoInline
  {
	 // Useful when debugging in GDB:
	 //System.out.println("About to bridge to 0x" + Integer.toHexString
	 //                   (VM_Magic.objectAsAddress(code).toInt()));

	 VM_Magic.dynamicBridgeTo(code);
	 // Never get here, dynamicBridgeTo returns to the thing that calls
	 // this method; the return value is provided by a suitable
	 // instruction in the VM_CodeArray.

	 // New pc value. Just to keep the compiler happy.
	 return 0xEBADC0DE;
  }
}

/**
 * This class is a hoax used to point our VM_PPC_Trace to. We can't
 * use DynamicCodeRunner as OPT_Compiler refuses to build something
 * that implements VM_DynamicBridge. VM_Uninterruptible is used to
 * prevent garbage collection errors with the dynamic bridge code.
 */
class DummyDynamicCodeRunner implements VM_Uninterruptible {
  public static int invokeCode (VM_CodeArray code, PPC_ProcessSpace ps) throws BadInstructionException, VM_PragmaNoInline
  {
	 // Fake out calls to PPC_ProcessSpace to forge some bytecode
	 // locations
	 ps.doSysCall(); // bytecode number -- 1
	 ps.recordUncaughtBclr(0,0); // bytecode number -- 7
	 ps.recordUncaughtBcctr(0,0); // bytecode number -- 13
	 new BadInstructionException(0,null); // bytecode number -- 21
	 throw new Error("This should never be executed");
  }
}
