/*
 * (C) Copyright The University of Manchester 2004
 */
package org.binarytranslator.PPC_Emulator;
import com.ibm.JikesRVM.DynamicCodeRunner;
import com.ibm.JikesRVM.VM_CodeArray;
import com.ibm.JikesRVM.VM_Uninterruptible;
import com.ibm.JikesRVM.VM_Magic;
import com.ibm.JikesRVM.VM_PragmaNoInline;

/** 
 * The runtime system for the PPC emulator.
 * 
 * @author Richard Matley, Ian Rogers
 * 
 */
public class Main
{
  /**
	* Usage
	*/
  public static void usage ()
  {
	 System.out.println("org.binarytranslator.PPC_Emulator.Main <PPC ELF binary> <args...>");
  }

  /**
	* The process space the emulation takes place in
	*/
  PPC_ProcessSpace ps;
  /**
	* Constructor - should only be run from main
	* @param args command line arguments
	*/
  private Main(String[] args) {
	 // Set up a PPC_ProcessSpace and load the PPC binary file into it.
	 if (PPC_Options.debugRuntime) {
		System.out.println("Main: loading " + args[0]);
	 }
	 try {
		ps = new PPC_ProcessSpace(args);
	 }
	 catch(java.io.IOException e) {
		usage();
		throw new Error("Main: File access error - " + args[0]);
	 }
	 if (PPC_Options.debugRuntime) {
		System.out.println("Main: created");
	 }
  }

  /**
	* Main method
	*
	* @param args command line arguments - arg[0] is the name of the PPC binary.
	*/
  public static void main(String[] args) {
	 // Process any arguments for the emulator
	 for (int i=0; i < args.length; i++) {
		if(args[i].startsWith("-X:ppc:")) {
		  if(args[i].startsWith("-X:ppc:debugInstr=true")) {
			 PPC_Options.debugInstr = true;
		  }
		  else if(args[i].startsWith("-X:ppc:debugRuntime=true")) {
			 PPC_Options.debugRuntime = true;
		  }
		  else if(args[i].startsWith("-X:ppc:debugSyscall=true")) {
			 PPC_Options.debugSyscall = true;
		  }
		  else if(args[i].startsWith("-X:ppc:debugSyscallMore=true")) {
			 PPC_Options.debugSyscallMore = true;
		  }
		  else if(args[i].startsWith("-X:ppc:globalBranchLevel=")) {
			 PPC_Options.globalBranchLevel = Integer.parseInt(args[i].substring(25));
		  }
		  else if(args[i].startsWith("-X:ppc:initialOptLevel=")) {
			 PPC_Options.initialOptLevel = Integer.parseInt(args[i].substring(23));
		  }
		  else if(args[i].startsWith("-X:ppc:instrOpt0=")) {
			 PPC_Options.instrOpt0 = Integer.parseInt(args[i].substring(17));
		  }
		  else if(args[i].startsWith("-X:ppc:instrOpt1=")) {
			 PPC_Options.instrOpt1 = Integer.parseInt(args[i].substring(17));
		  }
		  else if(args[i].startsWith("-X:ppc:instrOpt2=")) {
			 PPC_Options.instrOpt2 = Integer.parseInt(args[i].substring(17));
		  }
		  else if(args[i].startsWith("-X:ppc:singleInstrTranslation=true")) {
			 PPC_Options.singleInstrTranslation = true;
		  }
		  else if(args[i].startsWith("-X:ppc:resolveBranchesAtOnce=true")) {
			 PPC_Options.resolveBranchesAtOnce = true;
		  }
		  else if(args[i].startsWith("-X:ppc:resolveBranchesAtOnce=false")) {
			 PPC_Options.resolveBranchesAtOnce = false;
		  }
		  else if(args[i].startsWith("-X:ppc:resolveProceduresBeforeBranches=true")) {
			 PPC_Options.resolveProceduresBeforeBranches = true;
		  }
		  else if(args[i].startsWith("-X:ppc:resolveProceduresBeforeBranches=false")) {
			 PPC_Options.resolveProceduresBeforeBranches = false;
		  }
		  else {
			 throw new Error("Unrecongised PPC emulator option " + args[i]);
		  }
		}
		else {
		  if (i != 0) {
			 String new_args[] = new String[args.length-i];
			 for (int j=0; j < (args.length-i); j++) {
				new_args[j]=args[i+j];
			 }
			 args = new_args;
		  }
		  break;
		}
	 }	 
	 Main runtime = new Main(args);
	 if (PPC_Options.debugRuntime) {
		for (int i=0; i < args.length; i++) {
		  System.out.println("Main: argument " + i + " " + args[i]);
		}
	 }
	 runtime.run();
  }

  /**
	* Runtime loop, goes through the PPC binary and looks in the
	* Hashtable codeHash to see if we have already translated/compiled
	* this piece of code, if not it is compiled. The compiled code is
	* then run.
	*/
  public void run() {
	 if (PPC_Options.debugRuntime) {
		System.out.println("Main: run");
	 }
				
	 // The current block of compiled code. 
	 VM_CodeArray code;  

	 // The current program counter.
	 int pc = ps.pc;

	 if(PPC_Options.debugPS) {
		System.out.println("***** INITIAL PROCESS SPACE *****\n" + ps);
	 }

	 if(PPC_Options.debugStack)	{
		System.out.print("***** INITIAL STACK *****\n");
		ps.printStack();
	 }

	 try {
		while(ps.finished == false) {
		  if(PPC_Options.debugRuntime) {
			 System.out.println("Running PC=0x" + Integer.toHexString(pc));
		  }
		  // Get the compiled code
		  code = ps.getCodeForPC(pc);
		  // Run the compiled code.
		  pc = DynamicCodeRunner.invokeCode(code, ps);
		}
	 }
	 catch(BadInstructionException e) {
		System.out.println(e.toString());
	 }
  }
}

