/*
 * (C) Copyright The University of Manchester 2004
 */
package com.ibm.JikesRVM.opt.ir;
// N.B. Not part of the PPC_Emulator package as package access is
// needed to OPT_GenerationContext
import com.ibm.JikesRVM.opt.OPT_CompilerPhase;

/** 
 * The OPT_CompilerPhase which translates from PowerPC machine code
 * into HIR. It is used in PPC_OptimizationPlanner (in place of
 * OPT_ConvertBCtoHIR and OPT_OptimizationPlanner).
 * 
 * @author Richard Matley, Ian Rogers
 *
 */
public final class PPC_ConvertPPCtoHIR extends OPT_CompilerPhase
{


  public PPC_ConvertPPCtoHIR() {}

  /**
	* Return a String describing this compiler phase.
	*/
  public final String getName()
  {
	 return "Generate HIR from PPC machine code";
  }

  /** 
	* Generate HIR from a block of PPC machine code. 
	*/
  public final void perform(OPT_IR ir)
  {
	 OPT_GenerationContext gc = new OPT_GenerationContext(ir.method, ir.compiledMethod,
																			ir.options, ir.inlinePlan);
	 PPC2IR.generateHIR(gc);

	 ir.gc = gc;
	 ir.cfg = gc.cfg;
	 ir.regpool = gc.temps;

	 if(gc.allocFrame) {
		ir.stackManager.forceFrameAllocation();
	 }

	 // ir now contains well formed HIR, we hope.
	 ir.IRStage = OPT_IR.HIR;
	 ir.HIRInfo = new OPT_HIRInfo(ir);
	 if(OPT_IR.SANITY_CHECK) {
		ir.verify("Initial HIR", true);
	 }

    //-#if RVM_FOR_POWERPC
	 PPC2IR.eliminateConditionalMoves(ir);
	 //-#endif
  }
}
