/*
 * (C) Copyright The University of Manchester 2004
 */
package org.binarytranslator.PPC_Emulator;

/**
 * Options controlling the PPC emulator
 */
public class PPC_Options {

  // -oO Translation settings Oo-

  /**
	* The initial optimisation level
	*/
  public static int initialOptLevel=0;

  /**
	* Instructions to translate for an optimisation level 0 trace
	*/
  public static int instrOpt0 = 684;

  /**
	* Instructions to translate for an optimisation level 1 trace
	*/
  public static int instrOpt1 = 1500;

  /**
	* Instructions to translate for an optimisation level 2 trace
	*/
  public static int instrOpt2 = 1500;


  /**
	* Favour backward branch optimization. Translate backward branch
	* addresses before the next instructions (this is the manner of the
	* 601's branch predictor).
	*/
  public final static boolean optimizeBackwardBranches = true;

  /**
	* Set this to true to record uncaught bclr instructions
	*/
  public static boolean plantUncaughtBclrWatcher = false;

  /**
	* Set this to true to record uncaught bcctr instructions
	*/
  public static boolean plantUncaughtBcctrWatcher = true;

  /**
	* Should all branches (excluding to lr and ctr) be resolved in one
	* big go or one at at a time
	*/
  public static boolean resolveBranchesAtOnce = true;

  /**
	* Should procedures (branches to ctr and lr) be given precedent over
	* more local branches
	*/
  public static boolean resolveProceduresBeforeBranches = true;

  /**
	* Use global branch information rather than local (within the
	* trace) information when optimisation level is greater than or
	* equal to this value
	*/
  public static int globalBranchLevel = 3;

  /**
	* Set this to true to translate only one PPC instruction at a
	* time.
	*/
  public static boolean singleInstrTranslation=false;

  /**
	* Translate load/store instructions as calls to methods in
	* PPCProcessSpace
	*/
  public final static boolean loadStoreInPS=false;

  /**
	* Eliminate unneeded filling of register
	*/
  public final static boolean eliminateRegisterFills = true;

  // -oO Translation debugging options Oo-

  /**
	* In PPC2IR, print dissassembly of PPC instructions.
	*/
  public static boolean debugInstr=false;

  /**
	* In PPC2IR, print information about lazy resolution...
	*/
  public final static boolean debugLazy=false;

  /**
	* In PPC2IR, print cfg.
	*/
  public final static boolean debugCFG=false;

  // -oO Runtime debugging options Oo-

  /**
	* In PPCProcessSpace, print syscall numbers.
	*/
  public static boolean debugSyscall=false;

  /**
	* In PPCProcessSpace, print syscall numbers.
	*/
  public static boolean debugSyscallMore=false;

  /**
	* In PPCEmulatorRuntime, print out various messages about the
	* emulator starting.
	*/
  public static boolean debugRuntime=false;

  /**
	* In PPCEmulatorRuntime, print out process space.
	*/
  public final static boolean debugPS=false;

  /**
	* In PPCEmulatorRuntime, when printing process space, omit FP
	* registers.
	*/
  public final static boolean debugPS_OmitFP=false;

  /** 
	* In PPCEmulatorRuntime, print out stack.
	*/
  public final static boolean debugStack=false;

  /**
	* Invocation count at which to start printing stack output,if DEBUG_STACK=true;
	*/
  public final static int debugStackInvMin=0;

  /** 
	* Invocation count at which to stop printing stack output.
	*/
  public final static int debugStackInvMax=1000;
}
