/*
 * (C) Copyright The University of Manchester 2004
 */
package com.ibm.JikesRVM.classloader;
// N.B. Not part of the PPC_Emulator package as protected fields are
// required from VM_NormalMethod
import org.binarytranslator.PPC_Emulator.*;
import com.ibm.JikesRVM.opt.*;
import com.ibm.JikesRVM.*;

/**
 * A method class which can be used in place of a VM_NormalMethod but
 * which includes a block of PPC machine code, since this is our
 * starting point for the PPC emulator, rather than Java byte
 * codes. Note this requires redefining the class VM_NormalMethod not
 * to be final.
 */
public final class VM_PPC_Trace extends VM_NormalMethod
{
  /*
	* Unique features of a PPC trace
	*/

  /**
	* The PPC_ProcessSpace within which we are running.
	*/
  public PPC_ProcessSpace ps;

  /**
	* Local copy of the program counter. In future developments
	* (parallel compilation) this might not be the same as ps.pc.
	*/
  public int pc; 

  /**
	* Traces appear as DynamicCodeRunner.invokeCode methods
	*/
  private static VM_Class dummyRunner;
  private static VM_NormalMethod invokeCode;
  private static VM_TypeReference dummyRunnerTypeRef;
  private static VM_MemberReference dummyRunnerMemRef;
  private static VM_Atom invokeCodeDescriptor;

  /**
	* PPC traces masquerade as the method used to invoke them. Generate
	* necessary references to it.
	*/
  static {
	 VM_Atom clsDescriptor = VM_Atom.findOrCreateAsciiAtom("Lcom/ibm/JikesRVM/DummyDynamicCodeRunner;");
	 VM_Atom memName       = VM_Atom.findOrCreateAsciiAtom("invokeCode");
	 invokeCodeDescriptor = VM_Atom.findOrCreateAsciiAtom("(Lcom/ibm/JikesRVM/VM_CodeArray;Lorg/binarytranslator/PPC_Emulator/PPC_ProcessSpace;)I");
	 VM_TypeReference tRef = VM_TypeReference.findOrCreate(VM_SystemClassLoader.getVMClassLoader(),
																			 clsDescriptor);
	 dummyRunnerTypeRef = VM_TypeReference.findOrCreate(VM_SystemClassLoader.getVMClassLoader(),
																		 clsDescriptor);
	 dummyRunnerMemRef = VM_MemberReference.findOrCreate(dummyRunnerTypeRef, memName, invokeCodeDescriptor);
	 if (dummyRunnerMemRef != null) {
		dummyRunner = (VM_Class)tRef.resolve();
		if (dummyRunner != null) {
		  dummyRunner.resolve();
		  invokeCode = (VM_NormalMethod)dummyRunner.findDeclaredMethod(memName, invokeCodeDescriptor);
		  if (invokeCode == null) {
			 throw new Error("Failed to find method " + memName + invokeCodeDescriptor + " in " + dummyRunner);
		  }
		}
		else {
		  throw new Error("Failed to resolve " + tRef);
		}
	 }
	 else {
		throw new Error("Failed to find " + memName);
	 }
  }

  /**
	* Constructor
	*
	* @param ps process space containing code
	* @param startPC the address of the first instruction
	*/
  public VM_PPC_Trace(PPC_ProcessSpace ps, int startPC)
  {
	 super(dummyRunner,
			 VM_MemberReference.findOrCreate(dummyRunnerTypeRef,
														VM_Atom.findOrCreateAsciiAtom("invokeCode" + "_PC_0x" + Integer.toHexString(startPC)),
														invokeCodeDescriptor),
			 invokeCode.modifiers,
			 invokeCode.getExceptionTypes(), 
			 invokeCode.getLocalWords(),
			 invokeCode.getOperandWords(),
			 invokeCode.bytecodes, 
			 invokeCode.getExceptionHandlerMap(),
			 null);

    this.offset = VM_Statics.allocateSlot(VM_Statics.METHOD) << LOG_BYTES_IN_INT;

	 this.summary |= HAS_ALLOCATION | HAS_THROW | HAS_INVOKE |
		HAS_FIELD_READ | HAS_FIELD_WRITE | HAS_ARRAY_READ | HAS_ARRAY_WRITE |
		HAS_COND_BRANCH | HAS_SWITCH | HAS_BACK_BRANCH |
		256;

	 this.ps = ps;
	 pc = startPC;

  }
}
